@@ -29,6 +29,7 @@
 #include "qgis.h"
 #include "qgsmapunitscale.h"
 #include "qgscolorramp.h"
+#include "qgsarrowsymbollayer.h"
 
 class QgsExpression;
 class QgsPathResolver;
@@ -54,6 +55,14 @@ class CORE_EXPORT QgsSymbolLayerUtils
 {
   public:
 
+    //! Editing vertex markers
+    enum VertexMarkerType
+    {
+      SemiTransparentCircle,
+      Cross,
+      NoMarker
+    };
+
     static QString encodeColor( const QColor &color );
     static QColor decodeColor( const QString &str );
 
@@ -87,6 +96,18 @@ class CORE_EXPORT QgsSymbolLayerUtils
     static QString encodeSldBrushStyle( Qt::BrushStyle style );
     static Qt::BrushStyle decodeSldBrushStyle( const QString &str );
 
+    /**
+     * Decodes a \a value representing an arrow head type.
+     * \since QGIS 3.2
+     */
+    static QgsArrowSymbolLayer::HeadType decodeArrowHeadType( const QVariant &value, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Decodes a \a value representing an arrow type.
+     * \since QGIS 3.2
+     */
+    static QgsArrowSymbolLayer::ArrowType decodeArrowType( const QVariant &value, bool *ok SIP_OUT = nullptr );
+
     /**
      * Encodes a QPointF to a string.
      * \see decodePoint()
@@ -101,6 +122,20 @@ class CORE_EXPORT QgsSymbolLayerUtils
      */
     static QPointF decodePoint( const QString &string );
 
+    /**
+     * Converts a \a value to a point.
+     *
+     * \param value value to convert
+     * \param ok if specified, will be set to TRUE if value was successfully converted
+     *
+     * \returns converted point
+     *
+     * \see decodePoint()
+     * \see toSize()
+     * \since QGIS 3.10
+     */
+    static QPointF toPoint( const QVariant &value, bool *ok SIP_OUT = nullptr );
+
     /**
      * Encodes a QSizeF to a string.
      * \see decodeSize()
@@ -117,6 +152,20 @@ class CORE_EXPORT QgsSymbolLayerUtils
      */
     static QSizeF decodeSize( const QString &string );
 
+    /**
+     * Converts a \a value to a size.
+     *
+     * \param value value to convert
+     * \param ok if specified, will be set to TRUE if value was successfully converted
+     *
+     * \returns converted size
+     *
+     * \see decodeSize()
+     * \see toPoint()
+     * \since QGIS 3.10
+     */
+    static QSizeF toSize( const QVariant &value, bool *ok SIP_OUT = nullptr );
+
     static QString encodeMapUnitScale( const QgsMapUnitScale &mapUnitScale );
     static QgsMapUnitScale decodeMapUnitScale( const QString &str );
 
@@ -165,7 +214,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \param padding space between icon edge and symbol
      * \see symbolPreviewPixmap()
      */
-    static QIcon symbolPreviewIcon( QgsSymbol *symbol, QSize size, int padding = 0 );
+    static QIcon symbolPreviewIcon( const QgsSymbol *symbol, QSize size, int padding = 0 );
 
     /**
      * Returns a pixmap preview for a color ramp.
@@ -173,10 +222,15 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \param size target pixmap size
      * \param padding space between icon edge and symbol
      * \param customContext render context to use when rendering symbol
+     * \param selected set to TRUE to render the symbol in a selected state
+     * \param expressionContext optional custom expression context
      * \note Parameter customContext added in QGIS 2.6
+     * \note Parameter selected added in QGIS 3.10
+     * \note Parameter expressionContext added in QGIS 3.10
      * \see symbolPreviewIcon()
      */
-    static QPixmap symbolPreviewPixmap( QgsSymbol *symbol, QSize size, int padding = 0, QgsRenderContext *customContext = nullptr );
+    static QPixmap symbolPreviewPixmap( const QgsSymbol *symbol, QSize size, int padding = 0, QgsRenderContext *customContext = nullptr, bool selected = false,
+                                        const QgsExpressionContext *expressionContext = nullptr );
 
     /**
      * Draws a symbol layer preview to a QPicture
@@ -185,10 +239,10 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \param size target size of preview picture
      * \param scale map unit scale for preview
      * \returns QPicture containing symbol layer preview
-     * \since QGIS 2.9
      * \see symbolLayerPreviewIcon()
+     * \since QGIS 2.9
      */
-    static QPicture symbolLayerPreviewPicture( QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit units, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale() );
+    static QPicture symbolLayerPreviewPicture( const QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit units, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale() );
 
     /**
      * Draws a symbol layer preview to an icon.
@@ -199,7 +253,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \returns icon containing symbol layer preview
      * \see symbolLayerPreviewPicture()
      */
-    static QIcon symbolLayerPreviewIcon( QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit u, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale() );
+    static QIcon symbolLayerPreviewIcon( const QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit u, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale() );
 
     /**
      * Returns an icon preview for a color ramp.
@@ -221,6 +275,12 @@ class CORE_EXPORT QgsSymbolLayerUtils
 
     static void drawStippledBackground( QPainter *painter, QRect rect );
 
+    /**
+     * Draws a vertex symbol at (painter) coordinates x, y. (Useful to assist vertex editing.)
+     * \since QGIS 3.4.5
+     */
+    static void drawVertexMarker( double x, double y, QPainter &p, QgsSymbolLayerUtils::VertexMarkerType type, int markerSize );
+
     //! Returns the maximum estimated bleed for the symbol
     static double estimateMaxSymbolBleed( QgsSymbol *symbol, const QgsRenderContext &context );
 
@@ -260,7 +320,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
     //! Reads and returns symbol layer from XML. Caller is responsible for deleting the returned object
     static QgsSymbolLayer *loadSymbolLayer( QDomElement &element, const QgsReadWriteContext &context ) SIP_FACTORY;
     //! Writes a symbol definition to XML
-    static QDomElement saveSymbol( const QString &symbolName, QgsSymbol *symbol, QDomDocument &doc, const QgsReadWriteContext &context );
+    static QDomElement saveSymbol( const QString &symbolName, const QgsSymbol *symbol, QDomDocument &doc, const QgsReadWriteContext &context );
 
     /**
      * Returns a string representing the symbol. Can be used to test for equality
@@ -369,7 +429,6 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \param doc The document owning the element
      * \param element The element parent
      * \param function The expression to be encoded
-     * \returns
      */
     static bool createExpressionElement( QDomDocument &doc, QDomElement &element, const QString &function );
     static bool createFunctionElement( QDomDocument &doc, QDomElement &element, const QString &function );
@@ -398,7 +457,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \see symbolFromMimeData()
      * \since QGIS 3.0
      */
-    static QMimeData *symbolToMimeData( QgsSymbol *symbol ) SIP_FACTORY;
+    static QMimeData *symbolToMimeData( const QgsSymbol *symbol ) SIP_FACTORY;
 
     /**
      * Attempts to parse \a mime data as a symbol. A new symbol instance will be returned
@@ -471,7 +530,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
     /**
      * Attempts to parse mime data as a color
      * \param data mime data to parse
-     * \param hasAlpha will be set to true if mime data was interpreted as a color containing
+     * \param hasAlpha will be set to TRUE if mime data was interpreted as a color containing
      * an explicit alpha value
      * \returns valid color if mimedata could be interpreted as a color, otherwise an
      * invalid color
@@ -490,18 +549,18 @@ class CORE_EXPORT QgsSymbolLayerUtils
     /**
      * Creates mime data from a list of named colors
      * \param colorList list of named colors
-     * \param allFormats set to true to include additional mime formats, include text/plain and application/x-color
+     * \param allFormats set to TRUE to include additional mime formats, include text/plain and application/x-color
      * \returns mime data containing encoded colors
      * \since QGIS 2.5
      */
-    static QMimeData *colorListToMimeData( const QgsNamedColorList &colorList, const bool allFormats = true ) SIP_FACTORY;
+    static QMimeData *colorListToMimeData( const QgsNamedColorList &colorList, bool allFormats = true ) SIP_FACTORY;
 
     /**
      * Exports colors to a gpl GIMP palette file
      * \param file destination file
      * \param paletteName name of palette, which is stored in gpl file
      * \param colors colors to export
-     * \returns true if export was successful
+     * \returns TRUE if export was successful
      * \see importColorsFromGpl
      */
     static bool saveColorsToGpl( QFile &file, const QString &paletteName, const QgsNamedColorList &colors );
@@ -509,7 +568,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
     /**
      * Imports colors from a gpl GIMP palette file
      * \param file source gpl file
-     * \param ok will be true if file was successfully read
+     * \param ok will be TRUE if file was successfully read
      * \param name will be set to palette name from gpl file, if present
      * \returns list of imported colors
      * \see saveColorsToGpl
@@ -520,7 +579,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * Attempts to parse a string as a color using a variety of common formats, including hex
      * codes, rgb and rgba strings.
      * \param colorStr string representing the color
-     * \param strictEval set to true for stricter color parsing rules
+     * \param strictEval set to TRUE for stricter color parsing rules
      * \returns parsed color
      * \since QGIS 2.3
      */
@@ -530,8 +589,8 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * Attempts to parse a string as a color using a variety of common formats, including hex
      * codes, rgb and rgba strings.
      * \param colorStr string representing the color
-     * \param containsAlpha if colorStr contains an explicit alpha value then containsAlpha will be set to true
-     * \param strictEval set to true for stricter color parsing rules
+     * \param containsAlpha if colorStr contains an explicit alpha value then containsAlpha will be set to TRUE
+     * \param strictEval set to TRUE for stricter color parsing rules
      * \returns parsed color
      * \since QGIS 2.3
      */
@@ -556,10 +615,10 @@ class CORE_EXPORT QgsSymbolLayerUtils
     //! Returns a point on the line from startPoint to directionPoint that is a certain distance away from the starting point
     static QPointF pointOnLineWithDistance( QPointF startPoint, QPointF directionPoint, double distance );
 
-    //! Return a list of all available svg files
+    //! Returns a list of all available svg files
     static QStringList listSvgFiles();
 
-    //! Return a list of svg files at the specified directory
+    //! Returns a list of svg files at the specified directory
     static QStringList listSvgFilesAt( const QString &directory );
 
     /**
@@ -579,23 +638,23 @@ class CORE_EXPORT QgsSymbolLayerUtils
     //! Calculate the centroid point of a QPolygonF
     static QPointF polygonCentroid( const QPolygonF &points );
 
-    //! Calculate a point within of a QPolygonF
-    static QPointF polygonPointOnSurface( const QPolygonF &points );
+    //! Calculate a point on the surface of a QPolygonF
+    static QPointF polygonPointOnSurface( const QPolygonF &points, QList<QPolygonF> *rings = nullptr );
 
     //! Calculate whether a point is within of a QPolygonF
     static bool pointInPolygon( const QPolygonF &points, QPointF point );
 
     /**
-     * Return a new valid expression instance for given field or expression string.
+     * Returns a new valid expression instance for given field or expression string.
      * If the input is not a valid expression, it is assumed that it is a field name and gets properly quoted.
-     * If the string is empty, returns null pointer.
+     * If the string is empty, returns NULLPTR.
      * This is useful when accepting input which could be either a non-quoted field name or expression.
      * \since QGIS 2.2
      */
     static QgsExpression *fieldOrExpressionToExpression( const QString &fieldOrExpression ) SIP_FACTORY;
 
     /**
-     * Return a field name if the whole expression is just a name of the field .
+     * Returns a field name if the whole expression is just a name of the field .
      *  Returns full expression string if the expression is more complex than just one field.
      *  Using just expression->expression() method may return quoted field name, but that is not
      *  wanted for saving (due to backward compatibility) or display in GUI.
@@ -642,7 +701,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
       * Merges the local scale limits, if any, with the ones already in the map, if any
       * \since QGIS 3.0
       */
-    static void mergeScaleDependencies( int mScaleMinDenom, int mScaleMaxDenom, QgsStringMap &props );
+    static void mergeScaleDependencies( double mScaleMinDenom, double mScaleMaxDenom, QgsStringMap &props );
 
     /**
      * Encodes a reference to a parametric SVG into SLD, as a succession of parametric SVG using URL parameters,
