@@ -21,20 +21,17 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '1edf372fb80f2a35c77b128dc143c0f3c97cd8f3'
-
 import re
 import os
 import warnings
 
 from qgis.PyQt import uic
-from qgis.PyQt.QtCore import QCoreApplication, QDir, pyqtSignal
+from qgis.PyQt.QtCore import QCoreApplication, QDir, pyqtSignal, QFileInfo
 from qgis.PyQt.QtWidgets import QDialog, QMenu, QAction, QFileDialog, QInputDialog
 from qgis.PyQt.QtGui import QCursor
 from qgis.gui import QgsEncodingSelectionDialog
-from qgis.core import (QgsDataSourceUri,
+from qgis.core import (QgsProcessing,
+                       QgsDataSourceUri,
                        QgsCredentials,
                        QgsExpression,
                        QgsSettings,
@@ -43,7 +40,8 @@ from qgis.core import (QgsDataSourceUri,
                        QgsProcessingOutputLayerDefinition,
                        QgsProcessingParameterDefinition,
                        QgsProcessingParameterFileDestination,
-                       QgsProcessingParameterFolderDestination)
+                       QgsProcessingParameterFolderDestination,
+                       QgsProcessingParameterVectorDestination)
 from processing.core.ProcessingConfig import ProcessingConfig
 from processing.tools.dataobjects import createContext
 from processing.gui.PostgisTableSelector import PostgisTableSelector
@@ -105,56 +103,61 @@ class DestinationSelectionPanel(BASE, WIDGET):
         self.destinationChanged.emit()
 
     def selectOutput(self):
-        if isinstance(self.parameter, QgsProcessingParameterFolderDestination):
-            self.selectDirectory()
-        else:
-            popupMenu = QMenu()
+        popupMenu = QMenu()
 
-            if not self.default_selection:
-                if self.parameter.flags() & QgsProcessingParameterDefinition.FlagOptional:
-                    actionSkipOutput = QAction(
-                        self.tr('Skip Output'), self.btnSelect)
-                    actionSkipOutput.triggered.connect(self.skipOutput)
-                    popupMenu.addAction(actionSkipOutput)
-
-                if isinstance(self.parameter, QgsProcessingParameterFeatureSink) \
-                        and self.parameter.supportsNonFileBasedOutput():
-                    # use memory layers for temporary layers if supported
-                    actionSaveToTemp = QAction(
-                        self.tr('Create Temporary Layer'), self.btnSelect)
-                else:
-                    actionSaveToTemp = QAction(
-                        self.tr('Save to a Temporary File'), self.btnSelect)
-                actionSaveToTemp.triggered.connect(self.saveToTemporary)
-                popupMenu.addAction(actionSaveToTemp)
-
-            actionSaveToFile = QAction(
-                QCoreApplication.translate('DestinationSelectionPanel', 'Save to File…'), self.btnSelect)
-            actionSaveToFile.triggered.connect(self.selectFile)
-            popupMenu.addAction(actionSaveToFile)
+        if not self.default_selection:
+            if self.parameter.flags() & QgsProcessingParameterDefinition.FlagOptional:
+                actionSkipOutput = QAction(
+                    self.tr('Skip Output'), self.btnSelect)
+                actionSkipOutput.triggered.connect(self.skipOutput)
+                popupMenu.addAction(actionSkipOutput)
 
             if isinstance(self.parameter, QgsProcessingParameterFeatureSink) \
                     and self.parameter.supportsNonFileBasedOutput():
-                actionSaveToGpkg = QAction(
-                    QCoreApplication.translate('DestinationSelectionPanel', 'Save to GeoPackage…'), self.btnSelect)
-                actionSaveToGpkg.triggered.connect(self.saveToGeopackage)
-                popupMenu.addAction(actionSaveToGpkg)
-                actionSaveToPostGIS = QAction(
-                    QCoreApplication.translate('DestinationSelectionPanel', 'Save to PostGIS Table…'), self.btnSelect)
-                actionSaveToPostGIS.triggered.connect(self.saveToPostGIS)
-                settings = QgsSettings()
-                settings.beginGroup('/PostgreSQL/connections/')
-                names = settings.childGroups()
-                settings.endGroup()
-                actionSaveToPostGIS.setEnabled(bool(names))
-                popupMenu.addAction(actionSaveToPostGIS)
+                # use memory layers for temporary layers if supported
+                actionSaveToTemp = QAction(
+                    self.tr('Create Temporary Layer'), self.btnSelect)
+            elif isinstance(self.parameter, QgsProcessingParameterFolderDestination):
+                actionSaveToTemp = QAction(
+                    self.tr('Save to a Temporary Directory'), self.btnSelect)
+            else:
+                actionSaveToTemp = QAction(
+                    self.tr('Save to a Temporary File'), self.btnSelect)
+            actionSaveToTemp.triggered.connect(self.saveToTemporary)
+            popupMenu.addAction(actionSaveToTemp)
 
-            actionSetEncoding = QAction(
-                QCoreApplication.translate('DestinationSelectionPanel', 'Change File Encoding ({})…').format(self.encoding), self.btnSelect)
-            actionSetEncoding.triggered.connect(self.selectEncoding)
-            popupMenu.addAction(actionSetEncoding)
+        if isinstance(self.parameter, QgsProcessingParameterFolderDestination):
+            actionSaveToFile = QAction(
+                QCoreApplication.translate('DestinationSelectionPanel', 'Save to Directory…'), self.btnSelect)
+            actionSaveToFile.triggered.connect(self.selectDirectory)
+        else:
+            actionSaveToFile = QAction(
+                QCoreApplication.translate('DestinationSelectionPanel', 'Save to File…'), self.btnSelect)
+            actionSaveToFile.triggered.connect(self.selectFile)
+        popupMenu.addAction(actionSaveToFile)
 
-            popupMenu.exec_(QCursor.pos())
+        if isinstance(self.parameter, QgsProcessingParameterFeatureSink) \
+                and self.parameter.supportsNonFileBasedOutput():
+            actionSaveToGpkg = QAction(
+                QCoreApplication.translate('DestinationSelectionPanel', 'Save to GeoPackage…'), self.btnSelect)
+            actionSaveToGpkg.triggered.connect(self.saveToGeopackage)
+            popupMenu.addAction(actionSaveToGpkg)
+            actionSaveToPostGIS = QAction(
+                QCoreApplication.translate('DestinationSelectionPanel', 'Save to PostGIS Table…'), self.btnSelect)
+            actionSaveToPostGIS.triggered.connect(self.saveToPostGIS)
+            settings = QgsSettings()
+            settings.beginGroup('/PostgreSQL/connections/')
+            names = settings.childGroups()
+            settings.endGroup()
+            actionSaveToPostGIS.setEnabled(bool(names))
+            popupMenu.addAction(actionSaveToPostGIS)
+
+        actionSetEncoding = QAction(
+            QCoreApplication.translate('DestinationSelectionPanel', 'Change File Encoding ({})…').format(self.encoding), self.btnSelect)
+        actionSetEncoding.triggered.connect(self.selectEncoding)
+        popupMenu.addAction(actionSetEncoding)
+
+        popupMenu.exec_(QCursor.pos())
 
     def saveToTemporary(self):
         if isinstance(self.parameter, QgsProcessingParameterFeatureSink) and self.parameter.supportsNonFileBasedOutput():
@@ -229,12 +232,12 @@ class DestinationSelectionPanel(BASE, WIDGET):
     def selectFile(self):
         file_filter = getFileFilter(self.parameter)
         settings = QgsSettings()
-        if isinstance(self.parameter, QgsProcessingParameterFeatureSink):
+        if isinstance(self.parameter, (QgsProcessingParameterFeatureSink, QgsProcessingParameterVectorDestination)):
             last_ext_path = '/Processing/LastVectorOutputExt'
-            last_ext = settings.value(last_ext_path, '.gpkg')
+            last_ext = settings.value(last_ext_path, '.{}'.format(self.parameter.defaultFileExtension()))
         elif isinstance(self.parameter, QgsProcessingParameterRasterDestination):
             last_ext_path = '/Processing/LastRasterOutputExt'
-            last_ext = settings.value(last_ext_path, '.tif')
+            last_ext = settings.value(last_ext_path, '.{}'.format(self.parameter.defaultFileExtension()))
         else:
             last_ext_path = None
             last_ext = None
@@ -243,7 +246,7 @@ class DestinationSelectionPanel(BASE, WIDGET):
         filters = file_filter.split(';;')
         try:
             last_filter = [f for f in filters if '*{}'.format(last_ext) in f.lower()][0]
-        except:
+        except IndexError:
             last_filter = None
 
         if settings.contains('/Processing/LastOutputPath'):
@@ -302,7 +305,7 @@ class DestinationSelectionPanel(BASE, WIDGET):
             else:
                 self.saveToTemporary()
         else:
-            if value == 'memory:':
+            if value in ('memory:', QgsProcessing.TEMPORARY_OUTPUT):
                 self.saveToTemporary()
             elif isinstance(value, QgsProcessingOutputLayerDefinition):
                 if value.sink.staticValue() in ('memory:', ''):
@@ -322,15 +325,27 @@ class DestinationSelectionPanel(BASE, WIDGET):
     def getValue(self):
         key = None
         if self.use_temporary and isinstance(self.parameter, QgsProcessingParameterFeatureSink):
-            key = 'memory:'
+            key = QgsProcessing.TEMPORARY_OUTPUT
         elif self.use_temporary and not self.default_selection:
-            key = self.parameter.generateTemporaryDestination()
+            key = QgsProcessing.TEMPORARY_OUTPUT
         else:
             key = self.leText.text()
 
         if not key and self.parameter.flags() & QgsProcessingParameterDefinition.FlagOptional:
             return None
 
+        if key and not key == QgsProcessing.TEMPORARY_OUTPUT \
+                and not key.startswith('memory:') \
+                and not key.startswith('ogr:') \
+                and not key.startswith('postgres:') \
+                and not key.startswith('postgis:'):
+            # output should be a file path
+            folder = QFileInfo(key).path()
+            if folder == '.':
+                # output name does not include a folder - use default
+                default_folder = ProcessingConfig.getSetting(ProcessingConfig.OUTPUT_FOLDER)
+                key = QDir(default_folder).filePath(key)
+
         if isinstance(self.parameter, QgsProcessingParameterFolderDestination):
             return key
 
