@@ -21,13 +21,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'April 2013'
 __copyright__ = '(C) 2013, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
-import os
-import configparser
-
 from qgis.core import (QgsApplication,
                        QgsProcessingAlgorithm,
                        QgsProcessingParameterEnum,
@@ -43,8 +36,13 @@ from qgis.utils import iface
 
 
 def algorithmHelp(id):
-    """Prints algorithm parameters with their types. Also
-    provides information about options if any.
+    """
+    Prints algorithm parameters with their types. Also
+    provides information about parameters and outputs,
+    and their acceptable values.
+
+    :param id: An algorithm's ID
+    :type id: str
     """
     alg = QgsApplication.processingRegistry().algorithmById(id)
     if alg is not None:
@@ -89,16 +87,45 @@ def algorithmHelp(id):
         print('Algorithm "{}" not found.'.format(id))
 
 
-def run(algOrName, parameters, onFinish=None, feedback=None, context=None):
-    """Executes given algorithm and returns its outputs as dictionary
-    object.
+def run(algOrName, parameters, onFinish=None, feedback=None, context=None, is_child_algorithm=False):
     """
-    return Processing.runAlgorithm(algOrName, parameters, onFinish, feedback, context)
+    Executes given algorithm and returns its outputs as dictionary object.
+
+    :param algOrName: Either an instance of an algorithm, or an algorithm's ID
+    :param parameters: Algorithm parameters dictionary
+    :param onFinish: optional function to run after the algorithm has completed
+    :param feedback: Processing feedback object
+    :param context: Processing context object
+    :param is_child_algorithm: Set to True if this algorithm is being run as part of a larger algorithm,
+    i.e. it is a sub-part of an algorithm which calls other Processing algorithms.
+
+    :returns algorithm results as a dictionary, or None if execution failed
+    :rtype: Union[dict, None]
+    """
+    if onFinish or not is_child_algorithm:
+        return Processing.runAlgorithm(algOrName, parameters, onFinish, feedback, context)
+    else:
+        # for child algorithms, we disable to default post-processing step where layer ownership
+        # is transferred from the context to the caller. In this case, we NEED the ownership to remain
+        # with the context, so that further steps in the algorithm have guaranteed access to the layer.
+        def post_process(_alg, _context, _feedback):
+            return
+
+        return Processing.runAlgorithm(algOrName, parameters, onFinish=post_process, feedback=feedback, context=context)
 
 
 def runAndLoadResults(algOrName, parameters, feedback=None, context=None):
-    """Executes given algorithm and load its results into QGIS project
+    """
+    Executes given algorithm and load its results into the current QGIS project
     when possible.
+
+    :param algOrName: Either an instance of an algorithm, or an algorithm's ID
+    :param parameters: Algorithm parameters dictionary
+    :param feedback: Processing feedback object
+    :param context: Processing context object
+
+    :returns algorithm results as a dictionary, or None if execution failed
+    :rtype: Union[dict, None]
     """
     if isinstance(algOrName, QgsProcessingAlgorithm):
         alg = algOrName
@@ -110,7 +137,8 @@ def runAndLoadResults(algOrName, parameters, feedback=None, context=None):
         if not param.name() in parameters:
             continue
 
-        if isinstance(param, (QgsProcessingParameterFeatureSink, QgsProcessingParameterVectorDestination, QgsProcessingParameterRasterDestination)):
+        if isinstance(param, (QgsProcessingParameterFeatureSink, QgsProcessingParameterVectorDestination,
+                              QgsProcessingParameterRasterDestination)):
             p = parameters[param.name()]
             if not isinstance(p, QgsProcessingOutputLayerDefinition):
                 parameters[param.name()] = QgsProcessingOutputLayerDefinition(p, QgsProject.instance())
@@ -118,13 +146,21 @@ def runAndLoadResults(algOrName, parameters, feedback=None, context=None):
                 p.destinationProject = QgsProject.instance()
                 parameters[param.name()] = p
 
-    return Processing.runAlgorithm(alg, parameters=parameters, onFinish=handleAlgorithmResults, feedback=feedback, context=context)
+    return Processing.runAlgorithm(alg, parameters=parameters, onFinish=handleAlgorithmResults, feedback=feedback,
+                                   context=context)
 
 
 def createAlgorithmDialog(algOrName, parameters={}):
-    """Creates and returns an algorithm dialog for the specified algorithm, prepopulated
+    """
+    Creates and returns an algorithm dialog for the specified algorithm, prepopulated
     with a given set of parameters. It is the caller's responsibility to execute
     and delete this dialog.
+
+    :param algOrName: Either an instance of an algorithm, or an algorithm's ID
+    :param parameters: Initial algorithm parameters dictionary
+
+    :returns algorithm results as a dictionary, or None if execution failed
+    :rtype: Union[dict, None]
     """
     if isinstance(algOrName, QgsProcessingAlgorithm):
         alg = algOrName.create()
@@ -132,7 +168,7 @@ def createAlgorithmDialog(algOrName, parameters={}):
         alg = QgsApplication.processingRegistry().createAlgorithmById(algOrName)
 
     if alg is None:
-        return False
+        return None
 
     dlg = alg.createCustomParametersWidget(iface.mainWindow())
 
@@ -145,10 +181,15 @@ def createAlgorithmDialog(algOrName, parameters={}):
 
 
 def execAlgorithmDialog(algOrName, parameters={}):
-    """Executes an algorithm dialog for the specified algorithm, prepopulated
+    """
+    Executes an algorithm dialog for the specified algorithm, prepopulated
     with a given set of parameters.
 
-    Returns the algorithm's results.
+    :param algOrName: Either an instance of an algorithm, or an algorithm's ID
+    :param parameters: Initial algorithm parameters dictionary
+
+    :returns algorithm results as a dictionary, or None if execution failed
+    :rtype: Union[dict, None]
     """
     dlg = createAlgorithmDialog(algOrName, parameters)
     if dlg is None:
